@@ -144,14 +144,17 @@ class ScalarField(DataFieldBase):
 
     @fill_in_docstring
     def laplace(
-        self, bc: "BoundariesData", out: Optional[ScalarField] = None, **kwargs
+        self,
+        bc: Optional[BoundariesData],
+        out: Optional[ScalarField] = None,
+        **kwargs,
     ) -> ScalarField:
         """apply Laplace operator and return result as a field
 
         Args:
             bc:
                 The boundary conditions applied to the field.
-                {ARG_BOUNDARIES}
+                {ARG_BOUNDARIES_OPTIONAL}
             out (ScalarField, optional):
                 Optional scalar field to which the  result is written.
             label (str, optional):
@@ -166,7 +169,10 @@ class ScalarField(DataFieldBase):
 
     @fill_in_docstring
     def gradient_squared(
-        self, bc: "BoundariesData", out: Optional[ScalarField] = None, **kwargs
+        self,
+        bc: Optional[BoundariesData],
+        out: Optional[ScalarField] = None,
+        **kwargs,
     ) -> ScalarField:
         r"""apply squared gradient operator and return result as a field
 
@@ -175,7 +181,7 @@ class ScalarField(DataFieldBase):
         Args:
             bc:
                 The boundary conditions applied to the field.
-                {ARG_BOUNDARIES}
+                {ARG_BOUNDARIES_OPTIONAL}
             out (ScalarField, optional):
                 Optional vector field to which the result is written.
             label (str, optional):
@@ -194,14 +200,17 @@ class ScalarField(DataFieldBase):
 
     @fill_in_docstring
     def gradient(
-        self, bc: "BoundariesData", out: Optional["VectorField"] = None, **kwargs
+        self,
+        bc: Optional[BoundariesData],
+        out: Optional["VectorField"] = None,
+        **kwargs,
     ) -> "VectorField":
         """apply gradient operator and return result as a field
 
         Args:
             bc:
                 The boundary conditions applied to the field.
-                {ARG_BOUNDARIES}
+                {ARG_BOUNDARIES_OPTIONAL}
             out (VectorField, optional):
                 Optional vector field to which the result is written.
             label (str, optional):
